SystemOrganization addCategory: #'NXTalk-ST-Kernel'!
SystemOrganization addCategory: #'NXTalk-Building'!
SystemOrganization addCategory: #'NXTalk-ST-Collections'!
SystemOrganization addCategory: #'NXTalk-ST-Tests'!
SystemOrganization addCategory: #'NXTalk-Building-Tests'!
SystemOrganization addCategory: #'NXTalk-User Interface'!
SystemOrganization addCategory: #'NXTalk-ST-Hardware'!
SystemOrganization addCategory: #'NXTalk-Connections'!

!String methodsFor: '*nxtalk' stamp: 'mab 6/6/2008 11:13'!
asNXTObject
	| string |
	string := NXTString new: self size.
	^ string replaceFrom: 1 to: self size with: self! !

RBProgramNodeVisitor subclass: #NXTAbstractCodeCompiler
	instanceVariableNames: 'literals bytecodes half arguments temporaries compilerContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTAbstractCodeCompiler class methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:54'!
bytecodes
	^ Dictionary new
		at: #nop put: 0;
		at: #send put: 1;
		at: #selfSend put: 2;
		at: #push put: 3;
		at: #popEmpty put: 4;
		at: #return put: 5;
		at: #pop put: 6;
		at: #dup put: 7;
		yourself! !

!NXTAbstractCodeCompiler class methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 10:48'!
specialObjects
	^ Dictionary new
		at: #self put: 0;
		at: #nil put: 1;
		at: #true put: 2;
		yourself! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 20:06'!
acceptAssignmentNode: anAssignmentNode
	| pushNeeded |
	self visitNode: anAssignmentNode value.
	pushNeeded := anAssignmentNode parent isSequence not or: [ self lastStatement ].
	pushNeeded ifTrue: [ self addBytecode: #dup ].
	self addBytecode: #pop.
	self addVariable: anAssignmentNode variable name.
	^ pushNeeded! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/15/2008 14:35'!
acceptBlockNode: aBlockNode
	| block |
	block := NXTBlockCompiler new
		compilerContext: self;
		compile: aBlockNode.
	self addBytecode: #push.
	self image addObject: block.
	self addLiteral: block.
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/10/2008 11:36'!
acceptCascadeNode: cascadeNode
	self visitNode: cascadeNode receiver.
	cascadeNode messages size - 1 timesRepeat: [ self addBytecode: #dup ].
	cascadeNode messages 
		do: [ :message | self visitNode: message ]
		separatedBy: [ self addBytecode: #popEmpty ].
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 11:13'!
acceptLiteralNode: aLiteralNode
	| literal |
	self addBytecode: #push.
	aLiteralNode value ifNil: [
		literal := self image specialObjectAt: #nil ].
	(aLiteralNode value isKindOf: Boolean) ifTrue: [
		aLiteralNode value 
			ifTrue: [ literal := self image specialObjectAt: #true ]
			ifFalse: [ literal := self image specialObjectAt: #false ]].
	aLiteralNode value isString ifTrue: [
		literal := aLiteralNode value asNXTObject.
		self image addObject: literal.
	].
	aLiteralNode value isInteger ifTrue: [
		literal := aLiteralNode value ].
	self addLiteral: literal.
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/10/2008 11:36'!
acceptMessageNode: aMessageNode
	self primitive ifNotNil: [ ^ self ].
	aMessageNode isCascaded not ifTrue: [self visitNode: aMessageNode receiver].
	aMessageNode arguments do: [:each | self visitNode: each].
	self addBytecode: #send.
	self addLiteral: (self image symbolAt: aMessageNode selector).
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/12/2008 15:21'!
acceptReturnNode: aReturnNode
	(aReturnNode value isVariable and: [ aReturnNode value name = 'self' ] and: [ self compilesBlock not]) ifFalse: [
		super acceptReturnNode: aReturnNode.
		self addBytecode: #return ].
	^ false! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 15:34'!
acceptVariableNode: variableNode
	self addBytecode: #push.
	self addVariable: variableNode name.
	^ true! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:38'!
addArgument: index
	self addBytecodeArgument: (index bitAnd: 16r1F)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:39'!
addBytecode: aSymbol
	| bytecode |
	bytecode := self class bytecodes at: aSymbol.
	"Bytecodes only have 4 bits, thus they get packed"
	half 
		ifFalse: [ self bytecodes add: bytecode << 4. half := true ]
		ifTrue: [ self bytecodes at: self bytecodes size put: (self bytecodes last bitOr: bytecode). half := false ].! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:39'!
addBytecodeArgument: aNumber
	"Arguments always have 8 bits, thus they could be split over byte boundaries"
	half 
		ifFalse: [ self bytecodes add: aNumber ]
		ifTrue: [ self bytecodes at: self bytecodes size put: (self bytecodes last bitOr: aNumber >> 4).
			self bytecodes add: (aNumber bitAnd: 16rF) << 4 ].! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:39'!
addInstVar: index
	self addBytecodeArgument: ((index bitAnd: 16r1F) bitOr: 1 << 5)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 19:16'!
addLiteral: literal
	self addBytecodeArgument: (16r80 bitOr: (self mapLiteral: literal))! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 11:04'!
addSpecialObject: index
	self addBytecodeArgument: ((index bitAnd: 16r1F) bitOr: 3 << 5)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:40'!
addTemporary: index
	self addBytecodeArgument: ((index bitAnd: 16r1F) bitOr: 2 << 5)! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 11:03'!
addVariable: aName
	self class specialObjects at: aName asSymbol ifPresent: [ :index |
		^ self addSpecialObject: index ].
	self ifArgument: aName doWithIndex: [ :index |
		^ self addArgument: index ].
	self ifTemporary: aName doWithIndex: [ :index |
		^ self addTemporary: index ].
	self ifInstVar: aName doWithIndex: [ :index |
		^ self addInstVar: index ].
	self ifNXTClass: aName doWithName: [ :newName |
		^ self addLiteral: (self image classAt: newName asSymbol) ].
	self error: 'Unknown variable'.! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 19:26'!
allArguments
	^ arguments! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 19:12'!
allTemporaries
	^ temporaries! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:12'!
arguments
	^ arguments! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:12'!
arguments: anObject
	arguments := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
bytecodes
	^ bytecodes! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
bytecodes: anObject
	bytecodes := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:17'!
compilerContext
	^ compilerContext! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:17'!
compilerContext: anObject
	compilerContext := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/12/2008 15:20'!
compilesBlock
	^ false! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:15'!
half
	^ half! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:15'!
half: anObject
	half := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:33'!
ifArgument: aName doWithIndex: aBlock
	| index |
	index := self allArguments indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 19:27'!
ifInstVar: aName doWithIndex: aBlock
	| index |
	index := self instVarNames indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/22/2008 15:58'!
ifNXTClass: aName doWithName: aBlock
	| class |
	class := [ Smalltalk at: aName asSymbol ] on: Error do: [ ^ nil ].
	(class inheritsFrom: NXTObject) 
		ifTrue: [ aBlock value: aName ]
		ifFalse: [ (aName startsWith: 'NXT') ifFalse: [ 
			self ifNXTClass: 'NXT', aName doWithName: aBlock ]].! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:13'!
ifTemporary: aName doWithIndex: aBlock
	| index |
	index := self allTemporaries indexOf: aName.
	index isZero ifFalse: [ aBlock value: index - 1 ].
	! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:17'!
image
	^ self compilerContext image! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 19:28'!
initialize
	literals := OrderedCollection new.
	bytecodes := OrderedCollection new.
	arguments := OrderedCollection new.
	temporaries := OrderedCollection new.
	half := false.! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:18'!
instVarNames
	^ self compilerContext instVarNames! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 20:05'!
lastStatement
	^ false! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
literals
	^ literals! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
literals: anObject
	literals := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:44'!
mapLiteral: aLiteral
	^ (self literals indexOf: aLiteral ifAbsent: [ self literals add: aLiteral. self literals size ]) - 1! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 15:37'!
primitive
	^ nil! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
temporaries
	^ temporaries! !

!NXTAbstractCodeCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:14'!
temporaries: anObject
	temporaries := anObject! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:44'!
visitArgument: argumentNode
	self arguments add: argumentNode name! !

!NXTAbstractCodeCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 17:47'!
visitTemporaryVariable: aNode
	self temporaries add: aNode name.! !

NXTAbstractCodeCompiler subclass: #NXTBlockCompiler
	instanceVariableNames: 'lastStatement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 20:04'!
acceptSequenceNode: aSequenceNode 
	self visitTemporaryVariables: aSequenceNode temporaries.
	aSequenceNode statements allButLastDo: [:each | (self visitNode: each) ifTrue: [self addBytecode: #popEmpty]].
	self lastStatement: true.
	self visitNode: (aSequenceNode statements last).
	half ifTrue: [ self addBytecode: #nop ].! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:27'!
allArguments
	^ self arguments, self compilerContext allArguments! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 19:27'!
allTemporaries
	^ self temporaries, self compilerContext allTemporaries! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/22/2008 14:37'!
compile: aBlockNode
	self visitArguments: aBlockNode arguments.
	self visitNode: aBlockNode body.
	^ (NXTImageCompiledBlock 
		bytecodes: self bytecodes
		literals: self literals)
		tempCount: self temporaries size! !

!NXTBlockCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/12/2008 15:20'!
compilesBlock
	^ true! !

!NXTBlockCompiler methodsFor: 'accessing' stamp: 'mab 5/7/2008 17:44'!
initialize
	super initialize.
	lastStatement := false.! !

!NXTBlockCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 20:04'!
lastStatement
	^ lastStatement! !

!NXTBlockCompiler methodsFor: 'accessing' stamp: 'mab 4/28/2008 20:04'!
lastStatement: anObject
	lastStatement := anObject! !

NXTAbstractCodeCompiler subclass: #NXTMethodCompiler
	instanceVariableNames: 'primitive'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 14:59'!
acceptMethodNode: aMethodNode
	aMethodNode pragmas do: [:each | self visitNode: each].
	self visitMethodArguments: aMethodNode arguments.
	self visitNode: aMethodNode body! !

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 15:25'!
acceptPragmaNode: aPragmaNode
	| pragma newMethodNode |
	pragma := aPragmaNode pragma.
	pragma keyword = #NXTPrimitive: ifTrue: [
		self primitive: (pragma argumentAt: 1) ].
	pragma keyword = #nxtTakeMethodFrom: ifTrue: [ 
		newMethodNode := SqueakParser parseMethod: ((Smalltalk at: (pragma argumentAt: 1))
			compiledMethodAt: (aPragmaNode parent selector )) getSource.
		aPragmaNode parent arguments: newMethodNode arguments.
		aPragmaNode parent body: newMethodNode body ].! !

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 17:07'!
acceptSequenceNode: aSequenceNode 
	self primitive ifNotNil: [ ^ nil ].
	self visitTemporaryVariables: aSequenceNode temporaries.
	aSequenceNode statements do: [:each | (self visitNode: each) ifTrue: [self addBytecode: #popEmpty]].
	half ifTrue: [ self addBytecode: #nop ].! !

!NXTMethodCompiler methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 18:29'!
compileMethod: aCompiledMethod
	| ast |
	ast := SqueakParser parseMethod: aCompiledMethod getSource.
	self visitNode: ast.
	self primitive ifNil: [ ^ (NXTImageCompiledMethod 
		bytecodes: self bytecodes 
		literals: self literals)
		tempCount: self temporaries size ].
	^ NXTImageCompiledMethod primitive: self primitive! !

!NXTMethodCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:37'!
primitive
	^ primitive! !

!NXTMethodCompiler methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:08'!
primitive: anObject
	primitive := anObject! !

!PositionableStream methodsFor: '*nxtalk' stamp: 'mab 4/12/2008 14:24'!
littleEndianWord
	self nextLittleEndianNumber: 2! !

!PositionableStream methodsFor: '*nxtalk' stamp: 'mab 4/12/2008 14:24'!
littleEndianWord: number
	self nextLittleEndianNumber: 2 put: number! !

Object subclass: #NXTClassCompiler
	instanceVariableNames: 'image compiledClass superclass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 16:21'!
compile: aClass
	| metaclass class |
	self compiledClass: aClass.
	self superclass: self compileSuperclass.
	metaclass := self createMetaclass.
	class := self createClass.
	self image classMapping at: class put: metaclass.
	^ class! !

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 15:14'!
compileSuperclass
	^ self compiledClass name = #NXTObject
		ifFalse: [ self image classAt: self compiledClass superclass name ]
		ifTrue: [ nil ]! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
compiledClass
	^ compiledClass! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
compiledClass: anObject
	compiledClass := anObject! !

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/5/2008 17:10'!
createClass
	| class |
	class := NXTClass new.
	self image addObject: class.
	class name: (self image symbolAt: self compiledClass name).
	class format: self compiledClass elementSize.
	self superclass isNil
		ifFalse: [ class superclass: self superclass ]
		ifTrue: [ class superclass: (self image specialObjectAt: #nil) ].
	self includeMethodDictionaryOf: self compiledClass in: class.
	self includeInstanceVariablesOf: self compiledClass in: class.
	^ class! !

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/5/2008 17:18'!
createMetaclass
	| metaclass |
	metaclass := NXTMetaclass new.
	self image addObject: metaclass.
	metaclass name: (self image symbolAt: (self compiledClass name, ' class') asSymbol).
	self superclass isNil
		ifFalse: [ metaclass superclass: (self image classMapping at: self superclass) ]
		ifTrue: [ metaclass superclass: (self image specialObjectAt: #nil) ].
	metaclass format: 2.
	self includeInstanceVariablesOf: self compiledClass class in: metaclass.
	self includeMethodDictionaryOf: self compiledClass class in: metaclass.
	^ metaclass! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
image
	^ image! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:21'!
image: anObject
	image := anObject! !

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 11:25'!
includeInstanceVariablesOf: aClass in: aNXTClass
	| instVarNames |
	instVarNames := self image newObject.
	aNXTClass instVarNames: instVarNames.
	aClass instVarNames do: [ :name |
		instVarNames instanceVariables add: (self image symbolAt: name asSymbol) ]! !

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 13:57'!
includeMethodDictionaryOf: aClass in: aNXTClass
	| methodDictionary method index selectors |
	index := 1.
	selectors := aClass selectors reject: [ :selector |
		(aClass organization categoryOfElement: selector) = #'nxt-exclude' ].
	methodDictionary := NXTArray new: selectors size * 2.
	self image addObject: methodDictionary.
	aNXTClass methodDictionary: methodDictionary.
	selectors do: [ :selector |
		method := NXTMethodCompiler new
			compilerContext: self;
			compileMethod: (aClass compiledMethodAt: selector).
		self image addObject: method.
		methodDictionary at: index put: (self image symbolAt: selector).
		methodDictionary at: index + 1 put: method.
		index := index + 2 ].! !

!NXTClassCompiler methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 10:49'!
includeSuperclass
	self compiledClass name = #NXTObject
		ifFalse: [ self classObject 
			superclass: (self image classAt: self compiledClass superclass name) ]
		ifTrue: [ self classObject 
			superclass: (self image specialObjectAt: #nil) ].! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 4/23/2008 19:22'!
instVarNames
	^ self compiledClass allInstVarNames! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:39'!
superclass
	^ superclass! !

!NXTClassCompiler methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:39'!
superclass: anObject
	superclass := anObject! !

Object subclass: #NXTConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Connections'!

NXTConnection subclass: #NXTSerialConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Connections'!

NXTConnection subclass: #NXTSocketConnection
	instanceVariableNames: 'socketStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Connections'!

!NXTSocketConnection class methodsFor: 'as yet unclassified' stamp: 'mab 6/19/2008 14:21'!
localhost
	^ self on: (SocketStream openConnectionToHostNamed: '127.0.0.1' port: 10000)! !

!NXTSocketConnection class methodsFor: 'as yet unclassified' stamp: 'mab 6/19/2008 14:18'!
on: socket
	^ self new socketStream: socket! !

!NXTSocketConnection methodsFor: 'accessing' stamp: 'mab 6/19/2008 14:18'!
socketStream
	^ socketStream! !

!NXTSocketConnection methodsFor: 'accessing' stamp: 'mab 6/19/2008 14:18'!
socketStream: anObject
	socketStream := anObject! !

NXTConnection subclass: #NXTUsbConnection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Connections'!

Object subclass: #NXTImage
	instanceVariableNames: 'objects specialObjects symbols classes paddings classMapping mapper finished'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImage class methodsFor: 'as yet unclassified' stamp: 'mab 6/19/2008 15:47'!
specialClasses
	^ {
	#NXTSmallInteger.
	#NXTString
	}! !

!NXTImage class methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 00:01'!
specialObjects
	^ {
	#System -> #NXTSystem.
	#nil -> #NXTUndefinedObject.
	#true -> #NXTTrue. 
	#false -> #NXTFalse.
	}! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 6/2/2008 19:15'!
addObject: anNXTObject
	self finished ifTrue: [ Error signal: 'Image is already finished' ].
	(anNXTObject isKindOf: NXTObject) 
		ifTrue: [ ^  self objects add: anNXTObject ]
		ifFalse: [ Error signal: 'Attempt to insert non-NXTObject into image' ]! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 17:20'!
classAt: aClassName
	^ classes at: aClassName ifAbsentPut: [ NXTClassCompiler new
		image: self;
		compile: (Smalltalk at: aClassName) ]! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:41'!
classMapping
	^ classMapping! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:41'!
classMapping: anObject
	classMapping := anObject! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 6/2/2008 19:14'!
finished
	^ finished! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 6/2/2008 19:14'!
finished: anObject
	finished := anObject! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 19:00'!
includeClasses
	| changed |
	changed := true.
	[ changed ] whileTrue: [
		changed := false.
		self objects do: [ :object |
			(object class inheritsFrom: NXTImageObject) ifTrue: [
				object nxtClass isSymbol ifTrue: [
					changed := true.
					object nxtClass: (self classAt: object nxtClass) ]]
				ifFalse: [ self classAt: object class name ]]].! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 18:03'!
includeDependentObjects
	self includeSpecialObjects.
	self includeSpecialClasses.
	self includeClasses.
	self includeMetaclass.! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 5/22/2008 16:38'!
includeMetaclass
	| metaclass |
	metaclass := self classAt: #NXTMetaclass.
	self objects do: [ :object |
		object isMeta ifTrue: [ self classMapping at: object put: metaclass ]].
	"NXTObject class superclass = NXTClass"
	(self classMapping at: (self classAt: #NXTObject)) superclass: (self classAt: #NXTClass).! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 15:02'!
includeSpecialClasses
	self class specialClasses do: [ :class |
		self classAt: class ].! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 00:32'!
includeSpecialObjects
	self class specialObjects do: [ :assoc |
		self specialObjectAt: assoc key ].! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 19:14'!
initialize
	objects := OrderedCollection new.
	specialObjects := Dictionary new.
	symbols := Dictionary new.
	classes := Dictionary new.
	paddings := 0.
	classMapping := Dictionary new.
	finished := false.! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 16:15'!
instanceFor: aClassName
	| object classObject |
	classObject := self classAt: aClassName.
	object := self newObject: (classObject allInstVarNames size).
	object nxtClass: classObject.
	^ object
	! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 4/28/2008 23:31'!
isSpecialObject: aSymbol
	^ self class specialObjects detect: [ :each |
		each key = aSymbol ] ifNone: [ false ].! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:53'!
mapper
	^ mapper! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:53'!
mapper: anObject
	mapper := anObject! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 10:08'!
newObject
	^ self newObjectWithType: NXTImageObject! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 10:07'!
newObject: aSize
	^ self newObject: aSize type: NXTImageObject! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 16:40'!
newObject: aSize type: aNXTImageType
	| object |
	object := (self newObjectWithType: aNXTImageType) instanceVariables: (Array new: aSize).
	object instanceVariables size do: [ :index |
		object instanceVariables at: index put: (self specialObjectAt: #nil) ].
	^ object! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 5/21/2008 10:23'!
newObjectWithType: aNXTImageType
	^ self addObject: aNXTImageType new! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
objects
	^ objects! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
objects: anObject
	objects := anObject! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 6/2/2008 19:31'!
specialObjectAt: aSymbol
	| specialObject specialClass|
	^ specialObjects at: aSymbol ifAbsentPut: [
		specialClass := Smalltalk at: (self specialObjectClassAt: aSymbol).
		specialObject := specialClass new.
		specialClass instVarNames do: [ :name |
			specialObject instVarNamed: name put: (self specialObjectAt: #nil) ].
		self addObject: specialObject ]! !

!NXTImage methodsFor: 'accessing special objects' stamp: 'mab 4/28/2008 23:36'!
specialObjectClassAt: aSymbol
	^ (self class specialObjects detect: [ :each | each key = aSymbol ]) value! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
specialObjects
	^ specialObjects! !

!NXTImage methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:06'!
specialObjects: anObject
	specialObjects := anObject! !

!NXTImage methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 11:14'!
symbolAt: aSymbol
	^ symbols at: aSymbol ifAbsentPut: [ self addObject: aSymbol asNXTObject ]! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 4/12/2008 14:25'!
writeHeaderTo: aByteStream
	aByteStream littleEndianWord: 16r4242.
	aByteStream littleEndianWord: self objects size.! !

!NXTImage methodsFor: 'private' stamp: 'mab 6/2/2008 18:04'!
writeImageToStream: aByteStream
	self writeHeaderTo: aByteStream.
	self writeSpecialObjectsTo: aByteStream.
	self writeObjectsTo: aByteStream.! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 6/6/2008 14:57'!
writeObjectsTo: aByteStream
	self objects do: [ :each | 
		each writeOn: aByteStream withImage: self.
		aByteStream position odd ifTrue: [ "padding"
			paddings := paddings + 1.
			aByteStream nextPut: 0 ]].! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 5/21/2008 14:59'!
writeSpecialObjectsTo: aByteStream
	aByteStream 
		littleEndianWord: (self mapper idFor: (self specialObjects at: #runSymbol)).
	self class specialObjects do: [ :objectToClass |
		aByteStream littleEndianWord: (self mapper idFor: (self specialObjectAt: objectToClass key)) ].
	self class specialClasses do: [ :class |
		aByteStream littleEndianWord: (self mapper idFor: (self classAt: class)) ].! !

!NXTImage methodsFor: 'serializing' stamp: 'mab 6/2/2008 19:15'!
writeToStream: aByteStream
	self includeDependentObjects.
	self finished: true.
	self mapper: (NXTObjectMapper on: self objects).
	self writeImageToStream: aByteStream.! !

Object subclass: #NXTImageBuilder
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageBuilder class methodsFor: 'as yet unclassified' stamp: 'mab 2/20/2008 02:52'!
standardCategories
	^ #('NXTalk-Kernel' 'NXTalk-Collections')! !

!NXTImageBuilder methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 17:17'!
helloWorldImageOn: aByteStream
	self image: NXTImage new.
	self image specialObjects at: #runSymbol put: (self image symbolAt: #run).
	self image writeToStream: aByteStream.

! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 02:50'!
image
	^ image! !

!NXTImageBuilder methodsFor: 'accessing' stamp: 'mab 2/20/2008 02:50'!
image: anObject
	image := anObject! !

Object subclass: #NXTObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

NXTObject subclass: #NXTBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 18:41'!
on: exception do: handlerBlock
	<NXTPrimitive: 13>! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 16:51'!
restart
	<NXTPrimitive: 5>
	self restart! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/26/2008 18:15'!
value
	<NXTPrimitive: 2>! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 5/22/2008 16:40'!
value: aValue
	<NXTPrimitive: 2>! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 5/22/2008 16:40'!
value: aValue1 value: aValue2
	<NXTPrimitive: 2>! !

!NXTBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 16:43'!
whileTrue: aBlock
	self value ifFalse: [ ^ nil ].
	aBlock value.
	self restart.! !

NXTObject subclass: #NXTBoolean
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTBoolean methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 14:20'!
ifFalse: falseBlock ifTrue: trueBlock
	^ self ifTrue: trueBlock ifFalse: falseBlock! !

NXTBoolean subclass: #NXTFalse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:28'!
& aBoolean
	^ self! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:16'!
ifFalse: aBlock
	^ aBlock value! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:16'!
ifTrue: aBlock
	^ nil! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 14:19'!
ifTrue: trueBlock ifFalse: falseBlock
	^ falseBlock value! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:28'!
not
	^ true! !

!NXTFalse methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 11:11'!
or: alternativeBlock
	^ alternativeBlock value! !

NXTBoolean subclass: #NXTTrue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:29'!
& aBoolean
	^ aBoolean! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:17'!
ifFalse: aBlock
	^ nil! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 23:17'!
ifTrue: aBlock
	^ aBlock value! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 14:19'!
ifTrue: trueBlock ifFalse: falseBlock
	^ trueBlock value! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:28'!
not
	^ false! !

!NXTTrue methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 11:11'!
or: alternativeBlock
	^ self! !

NXTObject subclass: #NXTButton
	instanceVariableNames: 'id'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Hardware'!

NXTObject subclass: #NXTClass
	instanceVariableNames: 'methodDictionary superclass name format instVarNames'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTClass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 15:55'!
basicNew
	<NXTPrimitive: 15>! !

!NXTClass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 16:56'!
basicNew: sizeRequested
	<NXTPrimitive: 16>! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 6/5/2008 17:06'!
format
	^ format! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 6/5/2008 17:06'!
format: anObject
	format := anObject! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:47'!
instVarNames
	^ instVarNames! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:47'!
instVarNames: anObject
	instVarNames := anObject! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/22/2008 16:35'!
isClass
	^ true! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:46'!
methodDictionary
	^ methodDictionary! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:46'!
methodDictionary: anObject
	methodDictionary := anObject! !

!NXTClass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 16:06'!
name
	^ name! !

!NXTClass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 16:06'!
name: aSymbol
	^ name := aSymbol! !

!NXTClass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 15:56'!
new
	^ self basicNew initialize! !

!NXTClass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 16:48'!
new: sizeRequested 
	^ (self basicNew: sizeRequested) initialize  ! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 6/6/2008 15:37'!
selectors
	| c |
	c := OrderedCollection new.
	self methodDictionary pairsDo: [ :selector :method |
		c add: selector ].
	^ c! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:45'!
superclass
	^ superclass! !

!NXTClass methodsFor: 'accessing' stamp: 'mab 5/21/2008 14:45'!
superclass: anObject
	superclass := anObject! !

NXTClass subclass: #NXTMetaclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTMetaclass methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 16:34'!
isMeta
	^ true! !

!NXTMetaclass methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 16:09'!
name
	^ 'a Metaclass'! !

NXTObject subclass: #NXTCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Collections'!

!NXTCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 15:09'!
select: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument. 
	Collect into a new collection like the receiver, only those elements for 
	which aBlock evaluates to true. Answer the new collection."

	| newCollection |
	newCollection := self species new.
	self do: [:each | (aBlock value: each) ifTrue: [ newCollection add: each]].
	^newCollection! !

NXTCollection subclass: #NXTSequenceableCollection
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Collections'!

NXTSequenceableCollection variableSubclass: #NXTArray
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Collections'!

!NXTArray methodsFor: 'nxt-exclude' stamp: 'mab 5/29/2008 11:46'!
computeSize
	^ self size! !

!NXTArray methodsFor: 'nxt-exclude' stamp: 'mab 5/29/2008 11:45'!
instanceVariables
	| c |
	c := OrderedCollection new.
	1 to: self size do: [ :index |
		c add: (self at: index) ].
	^ c! !

!NXTArray methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 14:58'!
size
	<NXTPrimitive: 17>
	^ super size! !

NXTSequenceableCollection variableSubclass: #NXTOrderedCollection
	instanceVariableNames: 'array firstIndex lastIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Collections'!

!NXTOrderedCollection class methodsFor: 'instance creation' stamp: 'mab 5/29/2008 15:59'!
new
	^ self new: 10! !

!NXTOrderedCollection class methodsFor: 'instance creation' stamp: 'mab 5/29/2008 16:00'!
new: anInteger 
	^ self basicNew setCollection: (NXTArray new: anInteger)! !

!NXTOrderedCollection methodsFor: 'adding' stamp: 'mab 6/9/2008 15:30'!
add: newObject
	^self addLast: newObject! !

!NXTOrderedCollection methodsFor: 'adding' stamp: 'mab 5/29/2008 15:48'!
addLast: newObject 
	"Add newObject to the end of the receiver. Answer newObject."

	lastIndex = array size ifTrue: [self makeRoomAtLast].
	lastIndex := lastIndex + 1.
	array at: lastIndex put: newObject.
	^ newObject! !

!NXTOrderedCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:49'!
at: anInteger 
	"Answer my element at index anInteger. at: is used by a knowledgeable
	client to access an existing element"

	(anInteger < 1 or: [anInteger + firstIndex - 1 > lastIndex])
		ifTrue: [self errorNoSuchElement]
		ifFalse: [^ array at: anInteger + firstIndex - 1]! !

!NXTOrderedCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:07'!
at: anInteger put: anObject 
	"Put anObject at element index anInteger. at:put: cannot be used to
	append, front or back, to an ordered collection; it is used by a
	knowledgeable client to replace an element."

	| index |
	index := anInteger asInteger.
	(index < 1 or: [index + firstIndex - 1 > lastIndex])
		ifTrue: [self errorNoSuchElement]
		ifFalse: [^array at: index + firstIndex - 1 put: anObject]! !

!NXTOrderedCollection methodsFor: 'adding' stamp: 'mab 5/29/2008 16:00'!
grow
	"Become larger. Typically, a subclass has to override this if the subclass
	adds instance variables."
	| newArray |
	newArray := NXTArray new: self size + self growSize.
	newArray replaceFrom: 1 to: array size with: array startingAt: 1.
	array := newArray! !

!NXTOrderedCollection methodsFor: 'adding' stamp: 'mab 5/29/2008 15:48'!
growSize
	^ array size max: 2! !

!NXTOrderedCollection methodsFor: 'adding' stamp: 'mab 5/29/2008 15:48'!
makeRoomAtLast
	| newLast delta |
	newLast := self size.
	array size - self size = 0 ifTrue: [self grow].
	(delta := firstIndex - 1) = 0 ifTrue: [^ self].
	"we might be here under false premises or grow did the job for us"
	1 to: newLast do:
		[:index |
		array at: index put: (array at: index + delta).
		array at: index + delta put: nil].
	firstIndex := 1.
	lastIndex := newLast! !

!NXTOrderedCollection methodsFor: 'private' stamp: 'mab 5/29/2008 15:53'!
reset
	firstIndex := array size // 3 max: 1.
	lastIndex := firstIndex - 1! !

!NXTOrderedCollection methodsFor: 'private' stamp: 'mab 5/29/2008 15:53'!
setCollection: anArray
	array := anArray.
	self reset! !

!NXTOrderedCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:52'!
size
	"Answer how many elements the receiver contains."

	^ lastIndex - firstIndex + 1! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:06'!
, other
	| oldSize newSize c |
	newSize := self size + other size.
	c := self class new: newSize.
	1 to: self size do: [ :index | c at: index put: (self at: index) ].
	oldSize := self size.
	1 to: other size do: [ :index | c at: index + oldSize put: (other at: index) ].
	^ c
	! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:30'!
= otherCollection 
	"Answer true if the receiver is equivalent to the otherCollection.
	First test for identity, then rule out different species and sizes of
	collections. As a last resort, examine each element of the receiver
	and the otherCollection."

	self == otherCollection ifTrue: [^ true].
	self species == otherCollection species ifFalse: [^ false].
	^ self hasEqualElements: otherCollection! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:14'!
copyFrom: start to: stop 
	"Answer a copy of a subset of the receiver, starting from element at 
	index start until element at index stop."

	| newSize |
	newSize := stop - start + 1.
	^(self species new: newSize)
		replaceFrom: 1
		to: newSize
		with: self
		startingAt: start! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/5/2008 14:36'!
do: aBlock 
	"Refer to the comment in Collection|do:."
	1 to: self size do:
		[:index | aBlock value: (self at: index)]! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:14'!
first: n
	"Answer the first n elements of the receiver.
	Raise an error if there are not enough elements."

	^ self copyFrom: 1 to: n! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:30'!
hasEqualElements: otherCollection
	"Answer whether the receiver's size is the same as otherCollection's
	size, and each of the receiver's elements equal the corresponding 
	element of otherCollection.
	This should probably replace the current definition of #= ."

	| size |
	"(otherCollection isKindOf: SequenceableCollection) ifFalse: [^ false]."
	(size := self size) = otherCollection size ifFalse: [^ false].
	1 to: size do:
		[:index |
		(self at: index) = (otherCollection at: index) ifFalse: [^ false]].
	^ true! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:39'!
pairsDo: aBlock 
	"Evaluate aBlock with my elements taken two at a time.  If there's an odd number of items, ignore the last one.  Allows use of a flattened array for things that naturally group into pairs.  See also pairsCollect:"

	1 to: self size // 2 do:
		[:index | aBlock value: (self at: 2 * index - 1) value: (self at: 2 * index)]! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/5/2008 16:14'!
replaceFrom: start to: stop with: replacement 
	"This destructively replaces elements from start to stop in the receiver. 
	Answer the receiver itself. Use copyReplaceFrom:to:with: for 
	insertion/deletion which may alter the size of the result."

	replacement size = (stop - start + 1)
		ifFalse: [self error: 'Size of replacement doesnt match'].
	^self replaceFrom: start to: stop with: replacement startingAt: 1! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 15:50'!
replaceFrom: start to: stop with: replacement startingAt: repStart 
	"This destructively replaces elements from start to stop in the receiver 
	starting at index, repStart, in the sequenceable collection, 
	replacementCollection. Answer the receiver. No range checks are 
	performed."

	| index repOff |
	repOff := repStart - start.
	index := start - 1.
	[(index := index + 1) <= stop]
		whileTrue: [self at: index put: (replacement at: repOff + index)]! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:12'!
startsWith: start 
	| comp |
	self size < start size ifTrue: [^ false].
	comp := true.
	(self first: start size) with: start
		do: [:ea :ea2 | ea = ea2 ifFalse: [comp := false]].
	^ comp! !

!NXTSequenceableCollection methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:12'!
with: otherCollection do: twoArgBlock 
	"Evaluate twoArgBlock with corresponding elements from this collection and otherCollection."
	otherCollection size = self size ifFalse: [self error: 'otherCollection must be the same size'].
	1 to: self size do:
		[:index |
		twoArgBlock value: (self at: index)
				value: (otherCollection at: index)]! !

NXTSequenceableCollection variableByteSubclass: #NXTString
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Collections'!

!NXTString class methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 16:38'!
cr
	| string |
	string := self new: 1.
	string at: 1 put: 10.
	^ string! !

!NXTString class methodsFor: 'nxt-exclude' stamp: 'mab 6/5/2008 17:12'!
elementSize
	^ 1! !

!NXTString methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:53'!
at: index
	<primitive: 63>
	<NXTPrimitive: 23>! !

!NXTString methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:53'!
at: index put: aCharacter
	<primitive: 64>
	<NXTPrimitive: 24>! !

!NXTString methodsFor: 'nxt-exclude' stamp: 'mab 6/5/2008 17:17'!
computeSize
	^ self size! !

!NXTString methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:23'!
print
	NXTSystem printString: self! !

!NXTString methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 16:33'!
printNl
	NXTSystem printString: self, self class cr! !

!NXTString methodsFor: 'nxt-exclude' stamp: 'mab 6/6/2008 14:36'!
printOn: aStream
	aStream nextPutAll: 'an NXTString('''.
	aStream nextPutAll: self.
	aStream nextPutAll: ''')'.! !

!NXTString methodsFor: 'nxt-exclude' stamp: 'mab 6/5/2008 17:24'!
writeDataOn: aByteStream withImage: image
	1 to: self size do: [ :index |
		aByteStream nextPut: (self at: index) asInteger ].! !

NXTString variableByteSubclass: #NXTSymbol
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Collections'!

!NXTSymbol methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 16:32'!
= aSymbol
	self == aSymbol ifTrue: [ ^ true ].
	^ false! !

!NXTSymbol methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:31'!
print
	'#' print. NXTSystem printString: self.! !

!NXTSymbol methodsFor: 'nxt-exclude' stamp: 'mab 6/6/2008 14:36'!
printOn: aStream
	aStream nextPutAll: 'an NXTSymbol(#'.
	aStream nextPutAll: self.
	aStream nextPut: $).! !

NXTObject subclass: #NXTCompiledMethod
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

NXTObject subclass: #NXTException
	instanceVariableNames: 'messageText'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTException class methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 15:55'!
signal: text
	self new signal: text! !

!NXTException methodsFor: 'accessing' stamp: 'mab 5/21/2008 18:21'!
messageText
	^ messageText! !

!NXTException methodsFor: 'accessing' stamp: 'mab 5/21/2008 18:21'!
messageText: anObject
	messageText := anObject! !

!NXTException methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 18:44'!
signal
	<NXTPrimitive: 14>! !

!NXTException methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 16:14'!
signal: text
	self messageText: text.
	self signal! !

NXTException subclass: #NXTTestFailure
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

NXTObject subclass: #NXTImageObject
	instanceVariableNames: 'nxtClass instanceVariables'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

NXTImageObject subclass: #NXTImageCompiledMethod
	instanceVariableNames: 'literals bytecodes primitive tempCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

NXTImageCompiledMethod subclass: #NXTImageCompiledBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTImageCompiledBlock methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 17:34'!
initialize
	super initialize.
	nxtClass := #NXTBlock! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/22/2008 14:14'!
bytecodes: bytecodes literals: literals
	^self new 
		bytecodes: bytecodes;
		literals: literals! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 14:57'!
elementSize
	^ 1! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/23/2008 18:50'!
fromSelector: aSelector class: aClass within: anNXTImage 
	| aCompiledMethod |
	aCompiledMethod := aClass compiledMethodAt: aSelector.
	^ NXTMethodCompiler new
		image: anNXTImage ;
		instVarNames: aClass allInstVarNames ;
		compileMethod: aCompiledMethod! !

!NXTImageCompiledMethod class methodsFor: 'as yet unclassified' stamp: 'mab 4/16/2008 14:41'!
primitive: aNumber
	^ self new primitive: aNumber! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
bytecodes
	^ bytecodes! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
bytecodes: anObject
	bytecodes := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 18:20'!
computeSize
	^ self primitive
		ifNil: [ 2 + self bytecodes size + (2 * self literals size) ]
		ifNotNil: [ 2 ]! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
literals
	^ literals! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/15/2008 18:49'!
literals: anObject
	literals := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:03'!
methodHeader
	^ self primitive 
		ifNil: [ (self literals size bitAnd: 16r7F) << 8 bitOr: self tempCount << 3 ]
		ifNotNil: [ 1 << 15 bitOr: (self primitive bitAnd: 16r7FFF) ]! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:40'!
primitive
	^ primitive! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/16/2008 14:40'!
primitive: anObject
	primitive := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:01'!
tempCount
	^ tempCount! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:01'!
tempCount: anObject
	tempCount := anObject! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 6/6/2008 10:51'!
writeDataOn: aByteStream withImage: image
	self writeMethodHeaderOn: aByteStream.
	self primitive ifNil: [
		self literals do: [ :literal | 
			literal asNXTObject writeAsReferenceOn: aByteStream within: image ].
		self bytecodes do: [ :bytecode | aByteStream nextPut: bytecode ]].! !

!NXTImageCompiledMethod methodsFor: 'accessing' stamp: 'mab 4/23/2008 16:03'!
writeMethodHeaderOn: aByteStream
	aByteStream littleEndianWord: self methodHeader.! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 5/21/2008 17:26'!
computeSize
	^ self instanceVariables size! !

!NXTImageObject methodsFor: 'as yet unclassified' stamp: 'mab 4/28/2008 17:34'!
initialize
	nxtClass := #NXTSystem.
	instanceVariables := OrderedCollection new.! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:03'!
instanceVariables
	^ instanceVariables! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:03'!
instanceVariables: anObject
	instanceVariables := anObject! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 4/28/2008 16:03'!
nxtClass
	^ nxtClass ifNil: [ #NXTSystem ]! !

!NXTImageObject methodsFor: 'accessing' stamp: 'mab 2/20/2008 03:04'!
nxtClass: anObject
	nxtClass := anObject! !

NXTObject subclass: #NXTMotor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Hardware'!

!NXTObject class methodsFor: 'nxt-exclude' stamp: 'mab 6/5/2008 17:11'!
elementSize
	^ 2! !

!NXTObject methodsFor: 'comparing' stamp: 'mab 6/6/2008 16:27'!
= other
	^ self == other! !

!NXTObject methodsFor: 'comparing' stamp: 'mab 6/6/2008 16:27'!
== other
	<NXTPrimitive: 7>
	^ self == other! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 10:38'!
asNXTObject
	^ self! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 15:33'!
at: anInteger
	<NXTPrimitive: 20>
	^ super at: anInteger! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 15:34'!
at: anInteger put: anObject
	<NXTPrimitive: 21>
	^ super at: anInteger put: anObject! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 17:36'!
class
	<NXTPrimitive: 12>! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 5/21/2008 10:26'!
computeSize
	^ self class allInstVarNames size! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 6/5/2008 17:11'!
header
	| header |
	"TODO this code assumes, size is below 2048"
	header := ((self class elementSize bitAnd: 3) << 12 bitOr: 1 << 11) bitOr: (self computeSize bitAnd: 16r7FF).
	^ header! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 15:56'!
initialize! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 5/21/2008 17:19'!
instanceVariables
	| c |
	c := OrderedCollection new.
	self class allInstVarNames doWithIndex: [ :name :index |
		c add: (self instVarAt: index) ].
	^ c! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/22/2008 16:35'!
isClass
	^ false! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 16:33'!
isMeta
	^ false! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 14:22'!
isNil
	^ false! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 5/21/2008 10:29'!
nxtClass
	^ self class name asSymbol! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 15:39'!
perform: aSymbol
	<NXTPrimitive: 25>! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 15:44'!
printNumber: aSmallInt
	<NXTPrimitive: 4>
	Transcript show: aSmallInt asString; cr.! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 15:45'!
printString: aString
	<NXTPrimitive: 0>
	Transcript show: aString; cr.! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 15:28'!
size
	<NXTPrimitive: 17>
	^ super size! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/29/2008 14:41'!
species
	^ self class! !

!NXTObject methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 14:23'!
value
	^ self! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 6/6/2008 10:39'!
writeAsReferenceOn: aByteStream within: image
	aByteStream littleEndianWord: (image mapper idFor: self)! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 5/21/2008 16:46'!
writeClassOn: aByteStream withImage: image
	| class |
	class := image classMapping at: self ifAbsent: [ self nxtClass isSymbol 
		ifTrue: [ image classAt: self nxtClass ]
		ifFalse: [ self nxtClass ]].
	aByteStream littleEndianWord: (image mapper idFor: class).! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 6/6/2008 10:38'!
writeDataOn: aByteStream withImage: image
	self instanceVariables do: [ :instanceVariable |
		instanceVariable asNXTObject writeAsReferenceOn: aByteStream within: image ].! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 5/21/2008 10:35'!
writeHeaderOn: aByteStream
	aByteStream littleEndianWord: self header.! !

!NXTObject methodsFor: 'nxt-exclude' stamp: 'mab 5/21/2008 14:55'!
writeOn: aByteStream withImage: image
	self writeHeaderOn: aByteStream.
	self writeClassOn: aByteStream withImage: image.
	self writeDataOn: aByteStream withImage: image.! !

NXTObject subclass: #NXTSmallInteger
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:40'!
* other
	<NXTPrimitive: 22>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 4/29/2008 15:25'!
+ other
	<NXTPrimitive: 3>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 15:24'!
- aNumber
	<NXTPrimitive: 19>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 15:20'!
// aNumber
	<NXTPrimitive: 18>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 5/16/2008 21:46'!
< other
	<NXTPrimitive: 11>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 11:07'!
<= other
	^ self < other or: [ self = other ]! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:49'!
> other
	^ (self <= other) not! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:38'!
bitAnd: other
	<NXTPrimitive: 8>! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 5/7/2008 17:55'!
isZero
	^ self = 0! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 15:26'!
max: aNumber
	self < aNumber
		ifTrue: [ ^ aNumber ]
		ifFalse: [ ^ self ]! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:53'!
print
	NXTSystem printNumber: self.! !

!NXTSmallInteger methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 14:08'!
to: stop do: aBlock 
	| nextValue |
	nextValue := self.
	[ nextValue <= stop ]
		whileTrue: 
			[ aBlock value: nextValue.
			nextValue := nextValue + 1 ]! !

NXTObject subclass: #NXTSystem
	instanceVariableNames: 'running enterPressed driving right left'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTSystem class methodsFor: 'as yet unclassified' stamp: 'mab 6/16/2008 13:29'!
enterButton
	^ 1! !

!NXTSystem class methodsFor: 'as yet unclassified' stamp: 'mab 6/16/2008 13:29'!
exitButton
	^ 2! !

!NXTSystem class methodsFor: 'as yet unclassified' stamp: 'mab 6/16/2008 13:29'!
leftButton
	^ 3! !

!NXTSystem class methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:53'!
printNumber: aSmallInt
	<NXTPrimitive: 4>
	Transcript show: aSmallInt asString; cr.! !

!NXTSystem class methodsFor: 'as yet unclassified' stamp: 'mab 6/6/2008 15:15'!
printString: aString
	<NXTPrimitive: 0>
	Transcript show: aString; cr.! !

!NXTSystem class methodsFor: 'as yet unclassified' stamp: 'mab 6/16/2008 13:29'!
rightButton
	^ 4! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 4/29/2008 17:36'!
buttonRead
	<NXTPrimitive: 6>! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 6/16/2008 15:19'!
checkEnterDown: key
	(key = self class enterButton) & enterPressed not ifTrue: [
		enterPressed := true ].! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 6/16/2008 15:19'!
checkEnterUp: key
	(key = self class enterButton) not & enterPressed ifTrue: [
		enterPressed := false.
		driving 
			ifFalse: [ self printString: 'Start driving'.
				self driveAhead.
				driving := true ]
			ifTrue: [  self printString: 'Stop driving'.
				self stopDriving.
				driving := false ]].! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 6/16/2008 15:19'!
checkLeft: key
	(key = self class leftButton) & left not ifTrue: [
		left := true.
		self driveLeft ].
	(key = self class leftButton) not & left ifTrue: [
		left := false.
		self driveAhead ].! !

!NXTSystem methodsFor: 'button reading' stamp: 'mab 6/16/2008 15:20'!
checkRight: key
	(key = self class rightButton) & right not & left not ifTrue: [
		right := true.
		self driveRight ].
	(key = self class rightButton) not & right ifTrue: [
		right := false.
		self driveAhead ].! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 6/16/2008 19:06'!
driveAhead
	self motor: 0 mode: 1.
	self motor: 1 mode: 1.
	self motor: 0 speed: 75.
	self motor: 1 speed: 75.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 6/16/2008 19:06'!
driveBack
	self motor: 0 mode: 1.
	self motor: 1 mode: 1.
	self motor: 0 speed: -75.
	self motor: 1 speed: -75.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 6/16/2008 19:06'!
driveLeft
	self motor: 0 mode: 0.
	self motor: 0 speed: 0.
	self motor: 1 mode: 1.
	self motor: 1 speed: 75.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 6/16/2008 19:06'!
driveRight
	self motor: 0 mode: 1.
	self motor: 0 speed: 75.
	self motor: 1 mode: 0.
	self motor: 1 speed: 0.! !

!NXTSystem methodsFor: 'initialize-release' stamp: 'mab 5/17/2008 10:07'!
initialize
	self stopDriving.
	running := true.
	enterPressed := false.
	driving := false.
	left := false.
	right := false.
	! !

!NXTSystem methodsFor: 'system' stamp: 'mab 5/21/2008 17:40'!
keyLoop
	| key |
	key := self buttonRead.
	key = self class exitButton ifTrue: [ running := false ].
	self checkEnterDown: key.
	self checkEnterUp: key.
	self checkLeft: key.
	self checkRight: key.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 14:51'!
motor: aNumber mode: aMode
	<NXTPrimitive: 9>! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 14:51'!
motor: aNumber speed: aSpeed
	<NXTPrimitive: 1>! !

!NXTSystem methodsFor: 'printing' stamp: 'mab 6/6/2008 15:53'!
printNumber: aSmallInt
	self class printNumber: aSmallInt! !

!NXTSystem methodsFor: 'printing' stamp: 'mab 6/9/2008 16:41'!
printString: aString
	aString printNl! !

!NXTSystem methodsFor: 'system' stamp: 'mab 6/19/2008 15:58'!
run
	self
		initialize;
		runTests;
		printString: 'Waiting for USB connection...'.
	[ NXTUsb isConnected not ] whileTrue: [ 10 ].
	self printString: 'Got one - waiting for a message'.
	self printString: NXTUsb read.
	self
		printString: 'Entering driving mode now:';
	  	printString: 'Enter to start and stop engine';
  		printString: 'Left/Right for steering'.
	[ running ] whileTrue: [ self keyLoop ].
	self printString: 'Goodbye!!'.! !

!NXTSystem methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 15:49'!
runTests
	NXTArrayTest new run.
	NXTBlockTest new run.
	NXTExecutionTest new run.
	NXTMetaclassesTest new run.! !

!NXTSystem methodsFor: 'driving' stamp: 'mab 5/15/2008 14:58'!
stopDriving
	self motor: 0 mode: 0.
	self motor: 1 mode: 0.
	self motor: 0 speed: 0.
	self motor: 1 speed: 0.! !

!NXTSystem methodsFor: 'time' stamp: 'mab 5/16/2008 21:32'!
time
	<NXTPrimitive: 10>! !

!NXTSystem methodsFor: 'system' stamp: 'mab 5/20/2008 11:50'!
timeLoop
	self buttonRead = 1 ifTrue: [ running := false ].
	self time < 15
		ifTrue: [ self driveAhead ]
		ifFalse: [ self time < 20
			ifTrue: [ self stopDriving ]
			ifFalse: [ self time < 30
				ifTrue: [ self driveBack ]
				ifFalse: [ self stopDriving. running := false ]]]! !

NXTObject subclass: #NXTTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

NXTTestCase subclass: #NXTArrayTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Tests'!

!NXTArrayTest methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 14:40'!
testSize
	self assert: (NXTArray new: 5) size = 5
	! !

NXTTestCase subclass: #NXTBlockTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Tests'!

!NXTBlockTest methodsFor: 'as yet unclassified' stamp: 'mab 6/3/2008 13:33'!
testTemporaryAssignmentFromBlock
	| temp |
	temp := 10.
	[ temp := 5 ] value.
	self assert: temp = 5.! !

NXTTestCase subclass: #NXTExecutionTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Tests'!

!NXTExecutionTest methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 16:10'!
helper: anObject
	^ anObject! !

!NXTExecutionTest methodsFor: 'as yet unclassified' stamp: 'mab 6/5/2008 15:43'!
testAutomaticEndOfExecutionWithRecursiveBlocks
	"If this does not crash, the test is ok"
	| array |
	array := NXTArray new: 10.
	1 to: 10 do: [ :index | array at: index put: index ].
	array do: [ :each | self helper: each ].! !

NXTTestCase subclass: #NXTMetaclassesTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Tests'!

!NXTMetaclassesTest methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 17:54'!
testBlubmon
	self assert: false.! !

!NXTTestCase methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 16:13'!
assert: aBoolean
	aBoolean ifFalse: [ NXTTestFailure new signal: 'Assertion failed' ].! !

!NXTTestCase methodsFor: 'as yet unclassified' stamp: 'mab 6/9/2008 16:39'!
run
	| testSelectors |
	testSelectors := self class selectors select: [ :each | each startsWith: #test ].
	('Testing class ' , self class name) printNl.
	testSelectors do: [ :testSelector |
		[ self perform: testSelector. '.' print ] on: NXTTestFailure do: [ :ex | 
			'F' print.
			testSelector print.
			ex messageText print ]].
	'' printNl.! !

NXTObject subclass: #NXTUndefinedObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Kernel'!

!NXTUndefinedObject methodsFor: 'as yet unclassified' stamp: 'mab 5/20/2008 14:22'!
isNil
	^ true! !

NXTObject subclass: #NXTUsb
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-ST-Hardware'!

!NXTUsb class methodsFor: 'as yet unclassified' stamp: 'mab 6/19/2008 15:10'!
isConnected
	<NXTPrimitive: 26>
	^ false! !

!NXTUsb class methodsFor: 'as yet unclassified' stamp: 'mab 6/19/2008 15:36'!
read
	"Blocking. Reads the next incoming package from the USB port"
	<NXTPrimitive: 27>! !

Object subclass: #NXTObjectMapper
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building'!

!NXTObjectMapper class methodsFor: 'as yet unclassified' stamp: 'mab 3/17/2008 17:15'!
on: objectList
	^ self new objects: objectList! !

!NXTObjectMapper methodsFor: 'as yet unclassified' stamp: 'mab 5/21/2008 16:38'!
idFor: anObject
	^ (self objects indexOf: anObject ifAbsent: [ self error: 'Non-mappable object' ]) - 1! !

!NXTObjectMapper methodsFor: 'accessing' stamp: 'mab 3/17/2008 17:15'!
objects
	^ objects! !

!NXTObjectMapper methodsFor: 'accessing' stamp: 'mab 3/17/2008 17:15'!
objects: anObject
	objects := anObject! !

!Symbol methodsFor: '*nxtalk' stamp: 'mab 6/6/2008 11:14'!
asNXTObject
	| symbol |
	symbol := NXTSymbol new: self size.
	^ symbol replaceFrom: 1 to: self size with: self! !

Model subclass: #NXTConnectDialog
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-User Interface'!

!NXTConnectDialog class methodsFor: 'as yet unclassified' stamp: 'mab 6/12/2008 13:18'!
open
	^ ToolBuilder open: self! !

!NXTConnectDialog methodsFor: 'as yet unclassified' stamp: 'mab 6/12/2008 13:22'!
buildWith: builder
	| windowSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'NXT - Transfer'.
	windowSpec extent: 200@100.
	windowSpec children: OrderedCollection new.
	windowSpec children add: (builder pluggableButtonSpec new
		model: self;
		label: 'Connect';
		action: #connect;
		frame: (0.8@0 corner: 0.95@0.1);
		yourself).
	^builder build: windowSpec! !

Model subclass: #NXTTransferTool
	instanceVariableNames: 'localSelection localClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-User Interface'!

!NXTTransferTool class methodsFor: 'as yet unclassified' stamp: 'mab 6/10/2008 13:16'!
open
	^ ToolBuilder open: self! !

!NXTTransferTool methodsFor: 'as yet unclassified' stamp: 'mab 6/10/2008 16:03'!
buildWith: builder
	| windowSpec |
	windowSpec := builder pluggableWindowSpec new.
	windowSpec model: self.
	windowSpec label: 'NXT - Transfer'.
	windowSpec extent: 600@360.
	windowSpec children: OrderedCollection new.
	windowSpec children add: (builder pluggableButtonSpec new
		model: self;
		label: 'Connect';
		action: #connect;
		frame: (0.8@0 corner: 0.95@0.1);
		yourself).
	windowSpec children add: (builder pluggableListSpec new
		model: self;
		list: #localClasses;
		getSelected: #localSelection;
		setSelected: #localSelection:;
		frame: (0@0.2 corner: 0.45@0.9);
		yourself).
	windowSpec children add: (builder pluggableButtonSpec new
		model: self;
		label: '>';
		frame: (0.45@0.45 corner: 0.55@0.55);
		yourself).
	windowSpec children add: (builder pluggableListSpec new
		model: self;
		list: #remoteClasses;
		frame: (0.55@0.2 corner: 1@0.9);
		yourself).
	^builder build: windowSpec! !

!NXTTransferTool methodsFor: 'actions' stamp: 'mab 6/12/2008 13:19'!
connect
	NXTConnectDialog open.! !

!NXTTransferTool methodsFor: 'as yet unclassified' stamp: 'mab 6/10/2008 16:06'!
localClasses
	^ localClasses ifNil: [ 
		localClasses := (NXTObject allSubclasses collect: [ :class | class name ]) asSortedCollection ]! !

!NXTTransferTool methodsFor: 'accessing' stamp: 'mab 6/10/2008 16:04'!
localSelection
	^ localSelection ifNil: [ localSelection := self localClasses first ]! !

!NXTTransferTool methodsFor: 'accessing' stamp: 'mab 6/10/2008 15:57'!
localSelection: anObject
	localSelection := anObject! !

!NXTTransferTool methodsFor: 'as yet unclassified' stamp: 'mab 6/10/2008 13:56'!
remoteClasses
	^ OrderedCollection with: 'Not connected yet'! !

!SmallInteger methodsFor: '*nxtalk' stamp: 'mab 6/6/2008 10:50'!
asNXTObject
	^ self! !

!SmallInteger methodsFor: '*nxtalk' stamp: 'mab 6/6/2008 14:51'!
writeAsReferenceOn: aByteStream within: image
	aByteStream littleEndianWord: (((self bitAnd: 16r8000) >> 1 bitOr: (self bitAnd: 16r3FFF)) bitOr: 16r8000)! !

TestCase subclass: #NXTImageTest
	instanceVariableNames: 'image'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NXTalk-Building-Tests'!

!NXTImageTest methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 18:17'!
setUp
	image := NXTImage new.
	image specialObjects at: #runSymbol put: (image symbolAt: #run).! !

!NXTImageTest methodsFor: 'as yet unclassified' stamp: 'mab 6/2/2008 19:17'!
testObjectCountDoesNotChangeAfterWrite
	self 
		shouldnt: [ image writeToStream: (MultiByteBinaryOrTextStream with: String new) binary ]
		raise: Exception.! !
